/*
 *  global.c - auxiliary routines, mainly gp screen management and config
 *
 *  Copyright (C) 1997-2004 John Coppens (john@jcoppens.com)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "support.h"
#include "global.h"
#include "main.h"

preferences pref = {
  ""
};

GList           *pagelist = NULL;
int		 page_nr = 0,
		 timer_id = 0,
		 debug = 0,
		 tmpf_fd = 0;
char		 tmpf[50] = "/tmp/cbrXXXXXX";
GnomeCanvas     *book_canvas = NULL;
GnomeCanvasItem *this_page = NULL;
char		*extens[] = {
		".jpg", ".jpeg", ".png", ".gif"
		},
		 all_extensions[] = "\\.jpg\\|\\.jpeg\\|\\.png\\|\\.gif";
int		 nr_extensions = 4,
		 ext_nr = 0;
double		 zoom_factor = 1.0;

void
save_config(void)
{
  char path[120];

  sprintf(path, "%s/global/lastbook", prog_name);
  gnome_config_set_string(path, pref.lastbook);

  sprintf(path, "%s/global/fixed_dir", prog_name);
  gnome_config_set_string(path, pref.fixeddir);

  sprintf(path, "%s/global/nextpage", prog_name);
  gnome_config_set_int(path, pref.nextpage);

  sprintf(path, "%s/global/prevpage", prog_name);
  gnome_config_set_int(path, pref.prevpage);

  sprintf(path, "%s/startup/width", prog_name);
  gnome_config_set_int(path, pref.initwidth);

  sprintf(path, "%s/startup/height", prog_name);
  gnome_config_set_int(path, pref.initheight);

  sprintf(path, "%s/global/nav_pos", prog_name);
  gnome_config_set_int(path, pref.nav_pos);

  sprintf(path, "%s/global/nav_hor", prog_name);
  gnome_config_set_int(path, pref.nav_hor);

  sprintf(path, "%s/global/dirmode", prog_name);
  gnome_config_set_int(path, pref.dirmode);

  gnome_config_sync();
}


void
load_config(void)
{
  char path[120], *p;
  int def;

  sprintf(path, "%s/global/lastbook=", prog_name);
  strcpy(pref.lastbook, p = gnome_config_get_string_with_default(path, &def));
  g_free(p);

  sprintf(path, "%s/global/fixed_dir=", prog_name);
  strcpy(pref.fixeddir, p = gnome_config_get_string_with_default(path, &def));
  g_free(p);

  sprintf(path, "%s/global/nextpage=0", prog_name);
  pref.nextpage = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/global/prevpage=0", prog_name);
  pref.prevpage = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/startup/width=480", prog_name);
  pref.initwidth = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/startup/height=640", prog_name);
  pref.initheight = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/global/nav_pos=%d", prog_name, NAV_SE);
  pref.nav_pos = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/global/nav_hor=%d", prog_name, FALSE);
  pref.nav_hor = gnome_config_get_int_with_default(path, &def);

  sprintf(path, "%s/global/dirmode=0", prog_name);
  pref.dirmode = gnome_config_get_int_with_default(path, &def);

  if (pref.dirmode == DIRM_FIXED)		// fixed startup directory
    strncpy(pref.lastbook, pref.fixeddir, sizeof(pref.lastbook));
}


void
show_config(GtkWidget *ref)
{
  GtkWidget *w;
  char bff[20];

  w = lookup_widget(ref, "conf_goto_entry");
  gtk_entry_set_text(GTK_ENTRY(w), pref.fixeddir);

  switch (pref.nextpage) {
    case 0: w = lookup_widget(ref, "fwd_top_rbtn"); break;
    case 1: w = lookup_widget(ref, "fwd_btm_rbtn"); break;
    case 2: w = lookup_widget(ref, "fwd_stay_rbtn"); break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);

  switch (pref.prevpage) {
    case 0: w = lookup_widget(ref, "back_top_rbtn"); break;
    case 1: w = lookup_widget(ref, "back_btm_rbtn"); break;
    case 2: w = lookup_widget(ref, "back_stay_rbtn"); break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);

  sprintf(bff, "%d", pref.initwidth);
  w = lookup_widget(ref, "conf_w_entry");
  gtk_entry_set_text(GTK_ENTRY(w), bff);

  sprintf(bff, "%d", pref.initheight);
  w = lookup_widget(ref, "conf_h_entry");
  gtk_entry_set_text(GTK_ENTRY(w), bff);

  switch (pref.dirmode) {
    case 0: w = lookup_widget(ref, "conf_prevdir_rbtn"); break;
    case 1: w = lookup_widget(ref, "conf_goto_rbtn"); break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);

  switch (pref.nav_pos) {
    case NAV_NE: w = lookup_widget(ref, "conf_nav_ne_rbtn"); break;
    case NAV_SE: w = lookup_widget(ref, "conf_nav_se_rbtn"); break;
    case NAV_NW: w = lookup_widget(ref, "conf_nav_nw_rbtn"); break;
    case NAV_SW: w = lookup_widget(ref, "conf_nav_sw_rbtn"); break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);

  printf("sh: %d\n", pref.nav_hor);
  switch (pref.nav_hor) {
    case FALSE:
      w = lookup_widget(ref, "conf_nav_vert_rbtn"); 
      break;
    case TRUE:
      w = lookup_widget(ref, "conf_nav_hor_rbtn");
      break;
  }
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
}


void
parse_config(GtkWidget *ref)
{
  GtkWidget *w;

  w = lookup_widget(ref, "conf_goto_entry");
  strncpy(pref.fixeddir,
		gtk_entry_get_text(GTK_ENTRY(w)),
		sizeof(pref.fixeddir));

  w = lookup_widget(ref, "fwd_top_rbtn");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
    pref.nextpage = 0;
  else {
    w = lookup_widget(ref, "fwd_btm_rbtn");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
      pref.nextpage = 1;
    else
      pref.nextpage = 2;
  }

  w = lookup_widget(ref, "back_top_rbtn");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
    pref.prevpage = 0;
  else {
    w = lookup_widget(ref, "back_btm_rbtn");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
      pref.prevpage = 1;
    else
      pref.prevpage = 2;
  }

  w = lookup_widget(ref, "conf_w_entry");
  pref.initwidth = atol(gtk_entry_get_text(GTK_ENTRY(w)));

  w = lookup_widget(ref, "conf_h_entry");
  pref.initheight = atol(gtk_entry_get_text(GTK_ENTRY(w)));

  w = lookup_widget(ref, "conf_prevdir_rbtn");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
    pref.dirmode = 0;
  else
    pref.dirmode = 1;

  w = lookup_widget(ref, "conf_nav_nw_rbtn");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
    pref.nav_pos = NAV_NW;
  else {
    w = lookup_widget(ref, "conf_nav_ne_rbtn");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
      pref.nav_pos = NAV_NE;
    else {
      w = lookup_widget(ref, "conf_nav_se_rbtn");
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
        pref.nav_pos = NAV_SE;
      else
        pref.nav_pos = NAV_SW;
    }
  }

  w = lookup_widget(ref, "conf_nav_hor_rbtn");
  pref.nav_hor = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  printf("parse: %d\n", pref.nav_hor);
}


void
strtolower(char *s)
{
  int c;

  for (c = 0; c < strlen(s); c++)
    if (isalpha(s[c]))
      s[c] = tolower(s[c]);
}


gint
compare_pages(gconstpointer s1, gconstpointer s2)
{
  return strcmp(s1, s2);
}


void
initial_setup(void)
{
  book_canvas = GNOME_CANVAS(lookup_widget(MainWindow, "main_canvas"));

  if ((tmpf_fd = mkstemp(tmpf)) == -1) {
    printf(_("Cannot generate temporary file %s\n"), tmpf);
    return;
  }
}


void
clear_pagelist(void)
{
  int c;

  for (c = 0; c < g_list_length(pagelist); c++)
    g_free(g_list_nth_data(pagelist, c));

  g_list_free(pagelist);
  pagelist = NULL;
}


void
start_show(void)
{
  GtkWidget *w;
  char bff[200], *p, *bffbeg;
  FILE *dirf;
  gboolean first = TRUE;
  int t;

  if (strstr(pref.lastbook, ".cbz") ||
      strstr(pref.lastbook, ".CBZ")) {
    sprintf(bff, "unzip -l \"%s\" | grep \"%s\" > %s",
		pref.lastbook, all_extensions, tmpf);
    pref.booktype = CBZ_FILE;
  } else {
    sprintf(bff, "unrar v \"%s\" | grep \"%s\" > %s",
                pref.lastbook, all_extensions, tmpf);
    pref.booktype = CBR_FILE;
  }
  system(bff);

  dirf = fopen(tmpf, "r");
  if (dirf == NULL) {
    printf(_("Cannot open temporary file %s for reading\n"), tmpfile);
    return;
  }

  if (pagelist) clear_pagelist();

  switch (pref.booktype) {
    case CBR_FILE:
      bffbeg = (char *)&bff[1]; break;
    case CBZ_FILE:
      bffbeg = (char *)&bff[28]; break;
  }

  while (fgets(bff, sizeof(bff), dirf) != NULL) {
    strtolower(bffbeg);
    if (first) {
      for (t = 0; t < nr_extensions; t++) 
        if (strstr(bffbeg, extens[t]) != NULL) break;
      if (t == nr_extensions) {		// then we don't know how to handle
        printf(_("Cannot find a suitable extension"));
        exit(1);
      }
      ext_nr = t;
      printf("Image extension: %s\n", extens[ext_nr]);
      first = FALSE;
    }
    p = strstr(bffbeg, extens[ext_nr]);
    if (p == NULL) break;
    *p = '\0';
    p = strdup(bffbeg);
    pagelist = g_list_append(pagelist, p);
  }

  pagelist = g_list_sort(pagelist, compare_pages);
  show_page(0);
  fclose(dirf);
}


void
scroll_to(double where)
{
  GtkWidget *wdg;
  GtkAdjustment *adj;

  if (debug) printf("[scroll] to %.2f\n");
  wdg = lookup_widget(MainWindow, "main_scroller");
  adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(wdg));
  if (debug)
    printf("[vscroll] val: %.2f max: %.2f min: %.2f\n",
	adj->value, adj->upper, adj->lower);
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),
		where*(adj->upper - adj->page_size));
  gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(wdg), adj);
}


void
show_page(int nr)
{
  GnomeCanvasGroup *grp = gnome_canvas_root(book_canvas);
  GdkPixbuf *pxm;
  char bff[200], *p;
  double w, h;

  p = (char *)g_list_nth_data(pagelist, nr);
  printf("Requesting page %d/%d (%s)\n", nr, g_list_length(pagelist), p);

  switch (pref.booktype) {
    case CBR_FILE:
      sprintf(bff, "unrar p -ierr -cl -- \"%s\" \"%s%s\" > %s",
		pref.lastbook,
		p, extens[ext_nr],
		tmpf);
      break;
    case CBZ_FILE:
      sprintf(bff, "unzip -p -C \"%s\" \"%s%s\" > %s",
                pref.lastbook,
                p, extens[ext_nr],
                tmpf);
      break;
  }
  system(bff);

  sprintf(bff, _("CBR Pager - %s"), (char *)g_list_nth_data(pagelist, nr));
  gtk_window_set_title(GTK_WINDOW(MainWindow), bff);

  pxm = gdk_pixbuf_new_from_file(tmpf, NULL);
  if (!pxm) 
    printf(_("Can't load the page (%s)\n"), tmpf);

  w = gdk_pixbuf_get_width(pxm);
  h = gdk_pixbuf_get_height(pxm);

  if (this_page)
    gtk_object_destroy(GTK_OBJECT(this_page));
  this_page = gnome_canvas_item_new(grp,
                gnome_canvas_pixbuf_get_type(),
                "pixbuf", pxm,
                "x", 0.0, "y", 0.0,
                "width", w, "height", h,
                NULL);
  gdk_pixbuf_unref(pxm);
  gnome_canvas_set_scroll_region(book_canvas,
                0.0, 0.0, w, h);
}


void
next_page(void)
{
  if (page_nr == (g_list_length(pagelist) - 1)) return;

  page_nr++;
  show_page(page_nr);
  switch (pref.nextpage) {
    case 0: scroll_to(0.0); break;
    case 1: scroll_to(1.0); break;
  }
}


void
previous_page(void)
{
  if (page_nr == 0) return;

  page_nr--;
  show_page(page_nr);
  switch (pref.prevpage) {
    case 0: scroll_to(0.0); break;
    case 1: scroll_to(1.0); break;
  }
}


void
goto_begin(void)
{
  page_nr = 0;
  show_page(page_nr);
}


void
goto_end(void)
{
  page_nr = g_list_length(pagelist) - 1;
  show_page(page_nr);
}


void
update_nav_position(void)
{
  int x, y, w, h, nh, nw;

  gtk_window_get_position(GTK_WINDOW(MainWindow), &x, &y);

  gtk_window_set_policy(GTK_WINDOW(nav_window), FALSE, FALSE, TRUE);
  gtk_toolbar_set_orientation(
	GTK_TOOLBAR(lookup_widget(nav_window, "nav_toolbar")),
	pref.nav_hor ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);

  gtk_window_get_size(GTK_WINDOW(MainWindow), &w, &h);
  gtk_window_get_size(GTK_WINDOW(nav_window), &nw, &nh);

  switch (pref.nav_pos) {
    case NAV_NW:
      gtk_window_move(GTK_WINDOW(nav_window), x+5, y+30);
      break;
    case NAV_NE:
      gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+30);
      break;
    case NAV_SE:
      if (pref.nav_hor)
        gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-12);
      else
        gtk_window_move(GTK_WINDOW(nav_window), x+w-nw-3, y+h-nh);
      break;
    case NAV_SW:
      if (pref.nav_hor)
        gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-12); // -nh+10);
      else
        gtk_window_move(GTK_WINDOW(nav_window), x+5, y+h-nh);
      break;
  }
}


gboolean
follow_timer(gpointer data)
{
  update_nav_position();
  return TRUE;
}


void
install_timer(void)
{
  timer_id = gtk_timeout_add(FOLLOW_TIMER, follow_timer, NULL);
}


void
set_zoom_factor(double factor)
{
  gnome_canvas_set_pixels_per_unit(book_canvas, factor);
  zoom_factor = factor;
}


void
clean_up(void)
{
  close(tmpf_fd);
  unlink(tmpf);
}
